package com.xhkjedu.smarking.service.report.generate;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.smarking.mapper.exam.MsClassStudentMapper;
import com.xhkjedu.smarking.mapper.exam.MsSubjectMapper;
import com.xhkjedu.smarking.mapper.paper.MsPaperQtypeQuestionMapper;
import com.xhkjedu.smarking.mapper.report.reportother.MsrReportparamMapper;
import com.xhkjedu.smarking.mapper.stupaper.MsPaperStudentMapper;
import com.xhkjedu.smarking.mapper.stupaper.MsPaperStudentQuestionMapper;
import com.xhkjedu.smarking.model.exam.MsClassStudent;
import com.xhkjedu.smarking.model.paper.MsPaper;
import com.xhkjedu.smarking.model.paper.MsPaperQtypeQuestion;
import com.xhkjedu.smarking.model.report.reportclass.MsrClassQtype;
import com.xhkjedu.smarking.model.report.reportclass.MsrClassQuestion;
import com.xhkjedu.smarking.model.report.reportclass.MsrClassQuestionObj;
import com.xhkjedu.smarking.model.report.reportother.MsrReportparam;
import com.xhkjedu.smarking.model.report.reportstu.MsrStudent;
import com.xhkjedu.smarking.model.report.reportsubject.MsrSubjectQuestionSection;
import com.xhkjedu.smarking.model.stupaper.MsPaperStudentQuestion;
import com.xhkjedu.smarking.utils.MarkingUtil;
import com.xhkjedu.smarking.vo.report.reportother.RankGroupVo;
import com.xhkjedu.smarking.vo.report.reportother.SzJsonVo;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description：试题对应报告内容
 * @Author: WN
 * @Date: 2024/12/10 10:18:26
 **/
@Service
public class ReportGenerateQuestionService {
    @Resource
    private MsPaperStudentQuestionMapper msPaperStudentQuestionMapper;
    @Resource
    private MsClassStudentMapper msClassStudentMapper;
    @Resource
    private MsSubjectMapper msSubjectMapper;
    @Resource
    private MsPaperQtypeQuestionMapper msPaperQtypeQuestionMapper;
    @Resource
    private MsrReportparamMapper msrReportparamMapper;
    @Resource
    private MsPaperStudentMapper msPaperStudentMapper;

    public void generateQuestion(Integer examid, Integer exammode) {
        // 查询所有学生试题
        List<MsPaperStudentQuestion> stuQuestions = msPaperStudentQuestionMapper.listAllStuQuesByExamid(examid);
        List<MsClassStudent> classStudents = msClassStudentMapper.listStudentByExamId(examid);
        List<MsPaper> subjects = msSubjectMapper.listReportSubject(examid, exammode);
        List<MsrStudent> stuPapers = msPaperStudentMapper.listPaperStudentByExamId(examid, null);
        // 获取考试参数
        List<MsrReportparam> params = msrReportparamMapper.listReportparamByRpbelong(examid, "quesummary", null);

        // 根据科目进行区分
        for (MsPaper subject : subjects) {
            String subjectid = subject.getSubjectid();
            // 获取科目下所有学生试题
            List<MsPaperStudentQuestion> subjectStuQuestions =
                    stuQuestions.stream().filter(stuQuestion -> stuQuestion.getSubjectid().equals(subjectid)).collect(Collectors.toList());
            // 科目下所有班级学生
            List<MsClassStudent> subjectClassStudents = classStudents.stream().filter(classStudent -> classStudent.getSubjectid().equals(subjectid)).collect(Collectors.toList());
            // 获取试卷中所有小题基础信息包含大题信息
            List<MsPaperQtypeQuestion> questions = msPaperQtypeQuestionMapper.listQuestionsForReport(subject.getMpid());
            // 获取科目参数
            List<MsrReportparam> subjectParams = params.stream().filter(p -> p.getSubjectid().equals(subjectid)).collect(Collectors.toList());
            // 获取学生科目得分
            List<MsrStudent> subjectStuPapers = stuPapers.stream().filter(p -> p.getSubjectid().equals(subjectid)).collect(Collectors.toList());
            // 获取学生科目得分-科目默认分段学生
            List<MsrSubjectQuestionSection> subjectSectionQuestions = subjectSectionForQuestion(subjectStuPapers, subject.getPscore());
            // 处理小题
            handleQuestion(subjectStuQuestions, subjectClassStudents, questions, subjectParams, subjectSectionQuestions, examid, subjectid);
            //处理题型
            handleQtype(subjectStuQuestions, subjectClassStudents, questions, subjectParams, examid, subjectid);
        }
    }

    // 单个试题分析
    private void handleQuestion(List<MsPaperStudentQuestion> subjectStuQuestions, List<MsClassStudent> subjectClassStudents, List<MsPaperQtypeQuestion> questions,
                                List<MsrReportparam> params, List<MsrSubjectQuestionSection> subjectSectionQuestions, Integer examid, String subjectid) {

        List<MsrClassQuestion> saveQuestions = new ArrayList<>();// 小题分析
        List<MsrClassQuestionObj> saveQuestionObjs = new ArrayList<>();// 小题-客观题选项分析
        List<MsrSubjectQuestionSection> saveSectionQuestions = new ArrayList<>();// 学科分段之间学生在该题的整体得分率

        // 处理小题信息
        for (MsPaperQtypeQuestion question : questions) {
            // 获取第一小题所有作答情况，全校作答情况
            List<MsPaperStudentQuestion> stuQuestions = subjectStuQuestions.stream().filter(q -> q.getMptqid().equals(question.getMptqid())).collect(Collectors.toList());
            int stunum = stuQuestions.size();// 学生数量
            Double sumQScore = question.getQscore() * stunum;// 小题得分
            List<Double> stuScores = stuQuestions.stream().map(MsPaperStudentQuestion::getStuscore).collect(Collectors.toList());// 所有学生得分

            Double maxScore = stuQuestions.stream().mapToDouble(MsPaperStudentQuestion::getStuscore).max().orElse(0.0);// 最高分
            Double minScore = stuQuestions.stream().mapToDouble(MsPaperStudentQuestion::getStuscore).min().orElse(0.0);// 最低分
            Double avgScore = MarkingUtil.calculateAverage(stuScores);// 平均分
            Double avgScoreRate = MarkingUtil.calculateRate(avgScore, sumQScore);// 平均得分率

            MsrClassQuestion schoolQ = new MsrClassQuestion();
            schoolQ.setClassid(0);// 全体班级
            schoolQ.setExamid(examid);
            schoolQ.setSubjectid(subjectid);
            schoolQ.setSchoolmaxscore(maxScore);
            schoolQ.setSchoolminscore(minScore);
            schoolQ.setSchoolavgscore(avgScore);
            schoolQ.setSchoolavgrate(avgScoreRate);
            // 全校班级试题情况
            MsrClassQuestion schoolQue = questionClass(stuQuestions, question, params, schoolQ);
            saveQuestions.add(schoolQue);

            // 班级试题--客观题选项情况
            MsrClassQuestionObj qobj = new MsrClassQuestionObj();
            if (question.getQtype() == 1) {
                qobj.setExamid(examid);
                qobj.setSubjectid(subjectid);
                qobj.setClassid(0);
                qobj.setMptqid(question.getMptqid());
                List<MsrClassQuestionObj> qobjList = questionXuanxiang(stuQuestions, qobj);
                saveQuestionObjs.addAll(qobjList);
            }

            // 学科分段之间学生在该题的整体得分率
            List<MsrSubjectQuestionSection> sectionQuestions = subjectSectionQuestion(stuQuestions, subjectSectionQuestions, question);
            saveSectionQuestions.addAll(sectionQuestions);


            // 班级试题分析
            // 获取所有班级
            List<Integer> classIds = subjectClassStudents.stream().map(MsClassStudent::getClassid).distinct().collect(Collectors.toList());
            for (Integer classid : classIds) {
                // 获取班级下学生id
                List<Integer> classStuIds =
                        subjectClassStudents.stream().filter(c -> c.getClassid().equals(classid)).map(MsClassStudent::getStudentid).collect(Collectors.toList());
                // 获取班级下学生试题
                List<MsPaperStudentQuestion> classStuQuestions = stuQuestions.stream().filter(q -> classStuIds.contains(q.getStudentid())).collect(Collectors.toList());
                schoolQ.setClassid(classid);
                MsrClassQuestion classQue = questionClass(classStuQuestions, question, params, schoolQ);
                saveQuestions.add(classQue);

                // 班级试题--客观题选项情况
                if (question.getQtype() == 1) {
                    qobj.setClassid(classid);
                    List<MsrClassQuestionObj> qobjList = questionXuanxiang(classStuQuestions, qobj);
                    saveQuestionObjs.addAll(qobjList);
                }
            }
        }
    }

    // 单个试题分析-学校、班级分析
    private MsrClassQuestion questionClass(List<MsPaperStudentQuestion> stuQuestions, MsPaperQtypeQuestion question, List<MsrReportparam> params, MsrClassQuestion schoolQ) {
        int stunum = stuQuestions.size();// 学生数量
        Double qScore = question.getQscore();// 试题分值
        Double sumQScore = question.getQscore() * stunum;// 小题得分
        List<Double> stuScores = stuQuestions.stream().map(MsPaperStudentQuestion::getStuscore).collect(Collectors.toList());// 所有学生得分
        Double sumStuScore = stuQuestions.stream().mapToDouble(MsPaperStudentQuestion::getStuscore).sum();// 综合

        Double maxScore;// 最高分
        Double minScore;// 最低分
        Double avgScore;// 平均分
        Double avgScoreRate;// 平均得分率
        if (schoolQ.getClassid() == 0) {
            // 代表是校级
            maxScore = schoolQ.getSchoolmaxscore();
            minScore = schoolQ.getSchoolminscore();
            avgScore = schoolQ.getSchoolavgscore();
            avgScoreRate = schoolQ.getSchoolavgrate();
        } else {
            // 指定班级
            maxScore = stuQuestions.stream().mapToDouble(MsPaperStudentQuestion::getStuscore).max().orElse(0.0);// 最高分
            minScore = stuQuestions.stream().mapToDouble(MsPaperStudentQuestion::getStuscore).min().orElse(0.0);// 最低分
            avgScore = MarkingUtil.calculateAverage(stuScores);// 平均分
            avgScoreRate = MarkingUtil.calculateRate(avgScore, sumQScore);// 平均得分率
        }
        Double stuScoreRate = MarkingUtil.calculateRate(sumStuScore, sumQScore);// 学生得分率
        MsrClassQuestion classQ = new MsrClassQuestion();
        classQ.setExamid(schoolQ.getExamid());
        classQ.setSubjectid(schoolQ.getSubjectid());
        classQ.setClassid(schoolQ.getClassid());// 班级id
        classQ.setMptqid(question.getMptqid());
        classQ.setQtype(question.getQtype());
        classQ.setQn(question.getMptqn() + "." + question.getQn());
        classQ.setFullscore(question.getQscore());
        classQ.setStunum(stunum);
        classQ.setClassmaxscore(maxScore);
        classQ.setClassminscore(minScore);
        classQ.setClassavgscore(avgScore);
        classQ.setClassavgrate(avgScoreRate);

        classQ.setSchoolmaxscore(schoolQ.getSchoolmaxscore());
        classQ.setSchoolminscore(schoolQ.getSchoolminscore());
        classQ.setSchoolavgscore(schoolQ.getSchoolavgscore());
        classQ.setSchoolavgrate(schoolQ.getSchoolavgrate());

        classQ.setScorerate(stuScoreRate);
        classQ.setScoreratec(MarkingUtil.sub(MarkingUtil.calculateRate(maxScore, qScore), MarkingUtil.calculateRate(minScore, qScore)));// 得分率差=最高得分率-最低得分率
        classQ.setNd(MarkingUtil.div(stuScoreRate, qScore));// 难度
        String[] ndStr = questionNdAndQfd(params, 1, classQ.getNd());
        classQ.setNdms(ndStr[0]);// 难度描述
        classQ.setNdfw(ndStr[1]);// 难度范围
        classQ.setQfd(MarkingUtil.qfd(stuScores));// 区分度
        String[] qfdStr = questionNdAndQfd(params, 2, classQ.getQfd());
        classQ.setQfdms(qfdStr[0]);// 区分度描述
        classQ.setQfdfw(qfdStr[1]);// 区分度范围
        classQ.setBzc(MarkingUtil.bzc(stuScores));// 标准差
        classQ.setCyxs(MarkingUtil.div(classQ.getBzc(), avgScore));// 差异系数
        classQ.setJbzs(MarkingUtil.jbzs(stuScores, stuScoreRate));// 鉴别指数
        // 获取所有满分学生
        List<MsPaperStudentQuestion> fullStuQuestions = stuQuestions.stream().filter(q -> q.getStuscore().equals(q.getQscore())).collect(Collectors.toList());
        String mfids = fullStuQuestions.stream().map(p -> p.getStudentid().toString()).collect(Collectors.joining(","));// 满分学生id
        classQ.setMfnum(fullStuQuestions.size());
        classQ.setMfrate(MarkingUtil.div(classQ.getMfnum(), stunum));////正确率、满分人数占比
        classQ.setMfids(mfids);
        // 获取零分学生
        List<MsPaperStudentQuestion> lfStuQuestions = stuQuestions.stream().filter(q -> q.getStuscore() == 0).collect(Collectors.toList());
        String lfids = lfStuQuestions.stream().map(p -> p.getStudentid().toString()).collect(Collectors.joining(","));// 零分学生id
        classQ.setLfnum(lfStuQuestions.size());
        classQ.setLfrate(MarkingUtil.div(lfStuQuestions.size(), stunum));////零分人数占比
        classQ.setLfids(lfids);
        return classQ;
    }

    // 单个试题分析-试题难度、区分度
    private String[] questionNdAndQfd(List<MsrReportparam> params, Integer rptype, Double nd) {
        // 获取难度或者区分度
        MsrReportparam ndParam = params.stream().filter(p -> p.getRptype().equals(rptype)).findFirst().orElse(null);// 难度
        String ndms = "";
        String ndfw = "";
        if (ndParam != null) {
            List<SzJsonVo> szList = JSON.parseArray(ndParam.getSzjson(), SzJsonVo.class);
            for (SzJsonVo sz : szList) {
                if (nd == 1 && sz.getMaxvalue() == 1) {
                    ndms = sz.getDjkey();
                    ndfw = "[" + sz.getMinvalue() + "~" + "1]";
                    break;
                } else if (nd == 0 && sz.getMinvalue() == 0) {
                    ndms = sz.getDjkey();
                    ndfw = "[0" + "~" + sz.getMaxvalue() + ")";
                    break;
                } else if (nd >= sz.getMinvalue() && nd < sz.getMaxvalue()) {
                    ndms = sz.getDjkey();
                    ndfw = "[" + sz.getMinvalue() + "~" + sz.getMaxvalue() + ")";
                    break;
                }
            }
        }
        return new String[]{ndms, ndfw};
    }

    // 单个试题分析-客观题选项情况
    private List<MsrClassQuestionObj> questionXuanxiang(List<MsPaperStudentQuestion> stuQuestions, MsrClassQuestionObj questionObj) {
        Map<String, List<MsPaperStudentQuestion>> optionMap = stuQuestions.stream().collect(Collectors.groupingBy(MsPaperStudentQuestion::getStuanswer));
        int totalNum = stuQuestions.size();
        List<MsrClassQuestionObj> qobjList = new ArrayList<>();
        for (Map.Entry<String, List<MsPaperStudentQuestion>> entry : optionMap.entrySet()) {
            String option = entry.getKey();
            List<MsPaperStudentQuestion> optionStuQuestions = entry.getValue();

            MsrClassQuestionObj qobj = new MsrClassQuestionObj();
            qobj.setExamid(questionObj.getExamid());
            qobj.setSubjectid(questionObj.getSubjectid());
            qobj.setClassid(questionObj.getClassid());
            qobj.setMptqid(questionObj.getMptqid());
            if (N_Utils.isNotEmpty(option)) {
                qobj.setXxda(option);
            } else {
                qobj.setXxda("未选");
            }
            qobj.setXxnum(optionStuQuestions.size());
            qobj.setXxrate(MarkingUtil.calculateRate(optionStuQuestions.size(), totalNum));
            String stuids = optionStuQuestions.stream().map(q -> q.getStudentid().toString()).collect(Collectors.joining(","));
            qobj.setStuids(stuids);
            qobjList.add(qobj);
        }
        return qobjList;
    }

    // 单个试题-科目分段-默认5分一段为后续试题在段上学生得分率做准备
    private List<MsrSubjectQuestionSection> subjectSectionForQuestion(List<MsrStudent> StuPapers, Double pscore) {
        // 把科目总分分段，默认5分一段
        List<RankGroupVo> fdList = MarkingUtil.fdScore(pscore, 5);
        List<MsrSubjectQuestionSection> sections = new ArrayList<>();
        MsrStudent fdStuPaper = StuPapers.get(0);
        for (RankGroupVo fd : fdList) {
            // 获取学生得分在指定区间的学生
            List<MsrStudent> fdStuPapers;
            if (fd.getMaxvalue() == pscore) {
                fdStuPapers = StuPapers.stream().filter(s -> s.getStuscore() >= fd.getMinvalue() && s.getStuscore() <= fd.getMaxvalue()).collect(Collectors.toList());
            } else {
                fdStuPapers = StuPapers.stream().filter(s -> s.getStuscore() >= fd.getMinvalue() && s.getStuscore() < fd.getMaxvalue()).collect(Collectors.toList());
            }

            MsrSubjectQuestionSection question = new MsrSubjectQuestionSection();
            question.setExamid(fdStuPaper.getExamid());
            question.setSubjectid(fdStuPaper.getSubjectid());
            question.setFdfw(fd.getGroupname());
            question.setFdnum(fdStuPapers.size());
            question.setFdids(fdStuPapers.stream().map(s -> s.getStudentid().toString()).collect(Collectors.joining(",")));
            sections.add(question);
        }
        return sections;
    }

    // 单个试题分析-科目分段区间得分率
    private List<MsrSubjectQuestionSection> subjectSectionQuestion(List<MsPaperStudentQuestion> stuQuestions, List<MsrSubjectQuestionSection> sections,
                                                                         MsPaperQtypeQuestion question) {
        List<MsrSubjectQuestionSection> rtnList = new ArrayList<>();
        for (MsrSubjectQuestionSection section : sections) {
            String[] userids = section.getFdids().split(",");
            List<String> useridList = Arrays.asList(userids);
            List<MsPaperStudentQuestion> sectionStuQuestions = stuQuestions.stream().filter(q -> useridList.contains(q.getStudentid().toString())).collect(Collectors.toList());
            // 获取总分
            double sumScore = sectionStuQuestions.stream().mapToDouble(MsPaperStudentQuestion::getStuscore).sum();
            int stuNum = sectionStuQuestions.size();
            // 获取平均分
            double avgScore = MarkingUtil.div(sumScore, stuNum);
            // 获取平均得分率
            double avgScoreRate = MarkingUtil.calculateRate(avgScore, question.getQscore());
            section.setAvgscore(avgScore);
            section.setAvgrate(avgScoreRate);
            section.setQntype(1);
            section.setMptqid(question.getMptqid());
            rtnList.add(section);
        }
        return rtnList;
    }

    // 题型分析
    private void handleQtype(List<MsPaperStudentQuestion> subjectStuQuestions,List<MsClassStudent> subjectClassStudents, List<MsPaperQtypeQuestion> questions,
                             List<MsrReportparam> params,Integer examid, String subjectid) {
        List<MsrClassQtype> saveQtypes = new ArrayList<>();//班级题型分析
        // 处理大题
        Map<Integer, List<MsPaperQtypeQuestion>> qtypeMap = questions.stream().collect(Collectors.groupingBy(MsPaperQtypeQuestion::getMptid));
        for (Map.Entry<Integer, List<MsPaperQtypeQuestion>> entry : qtypeMap.entrySet()) {
            List<MsPaperQtypeQuestion> qtypeQuestions = entry.getValue();
            Double fullScore = qtypeQuestions.stream().mapToDouble(MsPaperQtypeQuestion::getQscore).sum();// 题型满分

            // 获取所有题型下试题id集合
            List<Integer> mptqids = qtypeQuestions.stream().map(MsPaperQtypeQuestion::getMptqid).distinct().collect(Collectors.toList());
            // 获取题型下所有学生试题情况
            List<MsPaperStudentQuestion> stuQuestions = subjectStuQuestions.stream().filter(q -> mptqids.contains(q.getMptqid())).collect(Collectors.toList());
            //初始化信息
            MsrClassQtype schoolQtype = new MsrClassQtype();
            schoolQtype.setExamid(examid);
            schoolQtype.setSubjectid(subjectid);
            schoolQtype.setClassid(0);
            schoolQtype.setMptid(entry.getKey());
            schoolQtype.setMptqn(qtypeQuestions.get(0).getMptqn());
            schoolQtype.setFullscore(fullScore);

            //全校班级题型情况
            MsrClassQtype qtypeSchool = qtypeClass(stuQuestions, params, schoolQtype);
            saveQtypes.add(qtypeSchool);

            // 班级题型分析
            // 获取所有班级
            List<Integer> classIds = subjectClassStudents.stream().map(MsClassStudent::getClassid).distinct().collect(Collectors.toList());
            for (Integer classid : classIds) {
                schoolQtype.setClassid(classid);
                // 获取班级下学生id
                List<Integer> classStuIds =
                        subjectClassStudents.stream().filter(c -> c.getClassid().equals(classid)).map(MsClassStudent::getStudentid).collect(Collectors.toList());
                // 获取班级下学生试题
                List<MsPaperStudentQuestion> classStuQuestions = stuQuestions.stream().filter(q -> classStuIds.contains(q.getStudentid())).collect(Collectors.toList());
                 // 班级题型分析
                MsrClassQtype qtypeClass = qtypeClass(classStuQuestions, params, schoolQtype);
                saveQtypes.add(qtypeClass);
            }
        }
    }
    // 题型分析-学校、班级分析
    private MsrClassQtype qtypeClass(List<MsPaperStudentQuestion> stuQuestions, List<MsrReportparam> params,MsrClassQtype schoolQtype){
        // 获取所有学生id
        List<Integer> studentIds = stuQuestions.stream().map(MsPaperStudentQuestion::getStudentid).distinct().collect(Collectors.toList());
        int stuNum = stuQuestions.size();
        List<Double> stuScores = new ArrayList<>();
        int mfnum = 0;
        String mfids = "";
        int lfnum = 0;
        String lfids = "";
        for (Integer studentId : studentIds) {
            List<MsPaperStudentQuestion> stuQtype = stuQuestions.stream().filter(q -> studentId.equals(q.getMptqid())).collect(Collectors.toList());
            Double stuQtypeScore = stuQtype.stream().mapToDouble(MsPaperStudentQuestion::getStuscore).sum();
            stuScores.add(stuQtypeScore);
            if(schoolQtype.getFullscore() == stuQtypeScore){
                mfnum++;
                mfids += studentId + ",";
            }else{
                lfnum++;
                lfids += studentId + ",";
            }
        }

        Double stuSumScore = stuQuestions.stream().mapToDouble(MsPaperStudentQuestion::getStuscore).sum();// 学生总分
        Double avgScore = MarkingUtil.div(stuSumScore, stuNum);// 平均分

        MsrClassQtype classQtype = new MsrClassQtype();
        classQtype.setExamid(schoolQtype.getExamid());
        classQtype.setSubjectid(schoolQtype.getSubjectid());
        classQtype.setClassid(schoolQtype.getClassid());
        classQtype.setMptid(schoolQtype.getMptid());
        classQtype.setMptqn(schoolQtype.getMptqn());
        classQtype.setFullscore(schoolQtype.getFullscore());
        classQtype.setAvgscore(avgScore);
        classQtype.setBzc(MarkingUtil.bzc(stuScores));
        classQtype.setNd(MarkingUtil.div(avgScore, schoolQtype.getFullscore()));
        classQtype.setNdms(questionNdAndQfd(params, 1, schoolQtype.getNd())[0]);
        classQtype.setQfd(MarkingUtil.qfd(stuScores));
        classQtype.setQfdms(questionNdAndQfd(params, 2, schoolQtype.getQfd())[0]);
        classQtype.setMfnum(mfnum);
        classQtype.setMfrate(MarkingUtil.div(mfnum, stuNum));
        if(mfnum > 0){
            schoolQtype.setMfids(mfids.substring(0,mfids.length()-1));
        }
        classQtype.setMfids(mfids);
        classQtype.setLfnum(lfnum);
        classQtype.setLfrate(MarkingUtil.div(lfnum, stuNum));
        if(lfnum > 0){
            classQtype.setMfids(lfids.substring(0,lfids.length()-1));
        }

        return classQtype;
    }

}
